#include "orbiter.hpp"

#include <iostream>
#include <astro/stateVectorIndices.hpp>

Orbiter::Orbiter(Icosphere& sphere, Orbit& orbit, GLuint shaderProgram) :
    _sphere(sphere),
    _orbit(orbit),
    _widget(shaderProgram)
{
}

float Orbiter::getMeanAnomaly()
{
    float time = glfwGetTime();
    return time;
}

void Orbiter::cycleAnimation()
{
    _animation++;

    if (_animation > ANIM_ECCENTRICITY)
    {
        _animation = 0;
    }
}

glm::vec3 Orbiter::getPosition(const float time)
{
    std::vector<float> keplerianElements(6);
    _orbit.getElements(keplerianElements);

    //int animation = (int)(time/ORBITAL_PERIOD) % 2 == 1;

    if (_animation == ANIM_ORBITING)
        return _orbit.getPosition(time);

    // TODO: i want to modify the eccentricity of the orbit with a control,
    // not an automatic animation
    if (_animation == ANIM_ECCENTRICITY)
    {
        // TODO: what are these magic numbers
        float e = .25 + .2 * sin(time);
        keplerianElements[astro::eccentricityIndex] = e;
        // TODO: extract set from getter
        _orbit.setElements(keplerianElements);

        return _orbit.getPosition(0);
    }

    std::cerr << "unknown animation " << _animation << std::endl;
    throw 1;
}

void Orbiter::render(const float time)
{
    const float meanAnomaly = time;
    glm::vec3 pos = getPosition(meanAnomaly);

    // render orbit
    _orbit.render(time);

    // render widget
    glm::mat4 widgetMatrix = _orbit.getLookAlongMatrix(time);
    _widget.setModelMatrix(widgetMatrix);
    _widget.render(time);

    // render sphere
    _sphere.setPosition(pos);
    _sphere.render(time);
}

Orbiter::~Orbiter()
{
}
